import type { Express, Request, Response } from "express";
import { createServer, type Server } from "http";
import { storage } from "./storage";
import { insertIncidentSchema, updateIncidentSchema, InsertIncident } from "@shared/schema";
import { ZodError } from "zod";
import { fromZodError } from "zod-validation-error";
import multer from "multer";
import fs from "fs";
import path from "path";
import { parseCSV } from "./utils/csv-parser";

// Configure multer for file uploads
const upload = multer({ 
  dest: 'uploads/',
  limits: {
    fileSize: 10 * 1024 * 1024, // 10MB max file size
  },
  fileFilter: (req, file, cb) => {
    // Accept only CSV files
    if (file.mimetype === 'text/csv' || file.originalname.endsWith('.csv')) {
      cb(null, true);
    } else {
      cb(new Error('Only CSV files are allowed'));
    }
  }
});

export async function registerRoutes(app: Express): Promise<Server> {
  // Make sure uploads directory exists
  const uploadsDir = path.join(process.cwd(), 'uploads');
  if (!fs.existsSync(uploadsDir)) {
    fs.mkdirSync(uploadsDir, { recursive: true });
  }

  // Import CSV data (pasted in textarea)
  app.post("/api/import/csv", async (req: Request, res: Response) => {
    try {
      const { csvData } = req.body;
      
      if (!csvData) {
        return res.status(400).json({ message: "CSV data is required" });
      }
      
      // Process the CSV data
      const rows = csvData.split('\n');
      
      // Get headers (first row)
      const headers = rows[0].split(',').map((header: string) => header.trim());
      
      // Process each row (skip headers)
      const incidents: InsertIncident[] = [];
      
      for (let i = 1; i < rows.length; i++) {
        if (!rows[i].trim()) continue; // Skip empty rows
        
        const values = rows[i].split(',');
        const incident: Partial<InsertIncident> = {};
        
        // Map values to incident properties based on headers
        headers.forEach((header: string, index: number) => {
          const value = values[index]?.trim() || '';
          
          // Map headers to incident properties
          switch(header.toLowerCase()) {
            case 'incident number':
            case 'call no':
            case 'call number':
              incident.incidentNumber = value;
              break;
            case 'date':
              incident.date = value;
              break;
            case 'time':
              incident.time = value;
              break;
            case 'reference':
              incident.reference = value || null;
              break;
            case 'location':
            case 'address':
              incident.location = value;
              break;
            case 'town':
              incident.town = value || null;
              break;
            case 'station':
              incident.station = value || null;
              break;
            case 'officer in charge':
            case 'officer':
              incident.officerInCharge = value || null;
              break;
            case 'watch':
              incident.watch = value || null;
              break;
            case 'pumps':
              incident.pumps = parseInt(value) || 0;
              break;
            case 'water carrier':
              incident.waterCarrier = parseInt(value) || 0;
              break;
            case 'support pump':
              incident.supportPump = parseInt(value) || 0;
              break;
            case 'aerial ladder':
              incident.aerialLadder = parseInt(value) || 0;
              break;
            case 'rescue support':
              incident.rescueSupport = parseInt(value) || 0;
              break;
            case 'incident command':
              incident.incidentCommand = value || null;
              break;
            case 'incident type':
            case 'type':
              incident.incidentType = value || null;
              break;
            case 'notes':
              incident.notes = value || null;
              break;
            case 'safety check':
              incident.safetyCheck = value || null;
              break;
            case 'additional notes':
              incident.additionalNotes = value || null;
              break;
            case 'safety check complete':
              incident.safetyCheckComplete = value || null;
              break;
          }
        });
        
        // Only add if we have the required fields
        if (incident.incidentNumber && incident.date) {
          incidents.push(incident as InsertIncident);
        }
      }
      
      res.status(200).json({ 
        message: `Successfully imported ${incidents.length} incidents` 
      });
    } catch (error) {
      console.error('Error importing CSV data:', error);
      res.status(500).json({ message: 'Failed to import CSV data' });
    }
  });
  
  // Import CSV file
  app.post("/api/import/csv-file", upload.single('csvFile'), async (req: Request, res: Response) => {
    try {
      const file = req.file;
      
      if (!file) {
        return res.status(400).json({ message: "No file uploaded" });
      }
      
      // Get the file path
      const filePath = path.join(process.cwd(), file.path);
      
      // Parse the CSV file
      const incidents = await parseCSV(filePath);
      
      // Process each incident and add to storage
      for (const incident of incidents) {
        await storage.createIncident(incident as InsertIncident);
      }
      
      // Clean up the uploaded file
      fs.unlinkSync(filePath);
      
      res.status(200).json({ 
        message: "Data imported successfully", 
        count: incidents.length 
      });
    } catch (error) {
      console.error("Error importing CSV file:", error);
      res.status(500).json({ 
        message: "Failed to import data", 
        error: error instanceof Error ? error.message : "Unknown error" 
      });
    }
  });

  // Clear all incidents
  app.post("/api/incidents/clear", async (req: Request, res: Response) => {
    try {
      await storage.clearAllIncidents();
      res.status(200).json({ message: "All incident data has been cleared" });
    } catch (error) {
      console.error('Error clearing incidents:', error);
      res.status(500).json({ message: 'Failed to clear incident data' });
    }
  });
  // GET statistics for dashboard
  app.get("/api/stats", async (req: Request, res: Response) => {
    try {
      const stats = await storage.getStats();
      res.json(stats);
    } catch (error) {
      res.status(500).json({ error: "Failed to retrieve statistics" });
    }
  });

  // GET incidents list with pagination and filtering
  app.get("/api/incidents", async (req: Request, res: Response) => {
    try {
      const page = parseInt(req.query.page as string) || 1;
      const limit = parseInt(req.query.limit as string) || 10;
      
      // Extract filters from query params
      const filters: Record<string, any> = {};
      const filterableFields = [
        'incidentNumber', 'date', 'location', 'town',
        'station', 'officerInCharge', 'watch', 'incidentType'
      ];
      
      filterableFields.forEach(field => {
        if (req.query[field]) {
          filters[field] = req.query[field];
        }
      });
      
      const result = await storage.getIncidents(page, limit, filters);
      res.json(result);
    } catch (error) {
      res.status(500).json({ error: "Failed to retrieve incidents" });
    }
  });

  // GET single incident by ID
  app.get("/api/incidents/:id", async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      if (isNaN(id)) {
        return res.status(400).json({ error: "Invalid incident ID" });
      }
      
      const incident = await storage.getIncidentById(id);
      if (!incident) {
        return res.status(404).json({ error: "Incident not found" });
      }
      
      res.json(incident);
    } catch (error) {
      res.status(500).json({ error: "Failed to retrieve incident" });
    }
  });

  // POST create a new incident
  app.post("/api/incidents", async (req: Request, res: Response) => {
    try {
      const validatedData = insertIncidentSchema.parse(req.body);
      const incident = await storage.createIncident(validatedData);
      res.status(201).json(incident);
    } catch (error) {
      if (error instanceof ZodError) {
        const validationError = fromZodError(error);
        return res.status(400).json({ error: validationError.message });
      }
      res.status(500).json({ error: "Failed to create incident" });
    }
  });

  // PUT update an existing incident
  app.put("/api/incidents/:id", async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      if (isNaN(id)) {
        return res.status(400).json({ error: "Invalid incident ID" });
      }
      
      const validatedData = updateIncidentSchema.parse({ ...req.body, id });
      const incident = await storage.updateIncident(id, validatedData);
      
      if (!incident) {
        return res.status(404).json({ error: "Incident not found" });
      }
      
      res.json(incident);
    } catch (error) {
      if (error instanceof ZodError) {
        const validationError = fromZodError(error);
        return res.status(400).json({ error: validationError.message });
      }
      res.status(500).json({ error: "Failed to update incident" });
    }
  });

  // DELETE remove an incident
  app.delete("/api/incidents/:id", async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      if (isNaN(id)) {
        return res.status(400).json({ error: "Invalid incident ID" });
      }
      
      const success = await storage.deleteIncident(id);
      if (!success) {
        return res.status(404).json({ error: "Incident not found" });
      }
      
      res.status(204).send();
    } catch (error) {
      res.status(500).json({ error: "Failed to delete incident" });
    }
  });

  // Export incidents to CSV
  app.get("/api/export/csv", async (req: Request, res: Response) => {
    try {
      const csv = await storage.exportToCSV();
      
      res.setHeader('Content-Type', 'text/csv');
      res.setHeader('Content-Disposition', 'attachment; filename="incidents.csv"');
      res.send(csv);
    } catch (error) {
      res.status(500).json({ error: "Failed to export incidents" });
    }
  });

  const httpServer = createServer(app);
  return httpServer;
}
